import 'package:bubble/bubble.dart';
import 'package:feed_plugin/bloc/feed_cubit.dart';
import 'package:feed_plugin/models/post_comment_model.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_screenutil/screen_util.dart';
import 'package:timeago/timeago.dart' as timeago;

class MessagesItem extends StatefulWidget {
  const MessagesItem(
      {Key key,
      @required this.postCommentModel,
      @required this.isUserMassage,
      @required this.commentId})
      : super(key: key);

  final PostCommentModel postCommentModel;
  final bool isUserMassage;
  final int commentId;

  @override
  _MessagesItemState createState() => _MessagesItemState();
}

class _MessagesItemState extends State<MessagesItem> {
  bool deleteComment = false;
  FeedCubit feedCubit;
  List<String> _comments = [];
  void _addComments(String val) {
    setState(() {
      _comments.add(val);
    });
  }

  @override
  void initState() {
    super.initState();
    feedCubit = BlocProvider.of<FeedCubit>(context);
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final date2 = DateTime.now();
    final difference =
        date2.difference(this.widget.postCommentModel.createdAt).inMinutes;
    final fifteenAgo =
        new DateTime.now().subtract(new Duration(minutes: difference));

    return GestureDetector(
      onLongPress: () {
        if (this.widget.isUserMassage) {
          this.setState(() {
            deleteComment = true;
          });
          showDialog(
              context: context,
              barrierDismissible: false,
              builder: (BuildContext context) {
                var dialogContext = context;
                return AlertDialog(
                  content:
                      Text("Are you sure you want to delete this comment?"),
                  actions: [
                    TextButton(
                      onPressed: () {
                        this.setState(() {
                          deleteComment = false;
                        });
                        Navigator.of(dialogContext).pop();
                      },
                      child: Text("Cancel"),
                    ),
                    TextButton(
                      onPressed: () {
                        feedCubit.feedRepository
                            .removeComment(this.widget.commentId)
                            .then((value) => {
                                  this.setState(() {
                                    deleteComment = false;
                                  }),
                                  Navigator.of(dialogContext).pop(),
                                });
                      },
                      child: Text("Ok"),
                    ),
                  ],
                );
              });
        }
      },
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Column(
            children: [
              Padding(
                padding: EdgeInsets.only(left: ScreenUtil().setWidth(9)),
                child: ClipOval(
                  child: Image.network(
                    this.widget.postCommentModel.user.profileImg,
                    width: ScreenUtil().setWidth(36),
                    height: ScreenUtil().setHeight(26),
                    fit: BoxFit.cover,
                  ),
                ),
              ),
            ],
          ),
          Padding(
            padding: EdgeInsets.only(left: ScreenUtil().setWidth(5)),
            child: Column(
              children: [
                Container(
                  width: 345,
                  decoration: BoxDecoration(
                    color: deleteComment ? Colors.grey : Color(0xffE8ECEF),
                    borderRadius: BorderRadius.circular(10),
                    boxShadow: [
                      BoxShadow(
                        color: Colors.white70,
                        blurRadius: 0,
                        offset: Offset(5, 5),
                      ),
                    ],
                  ),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Row(children: [
                        Padding(
                          padding: EdgeInsets.only(
                              left: ScreenUtil().setWidth(15),
                              top: ScreenUtil().setWidth(9)),
                          child: Text(
                            this
                                    .widget
                                    .postCommentModel
                                    .user
                                    .firstName
                                    .toString() +
                                " " +
                                this
                                    .widget
                                    .postCommentModel
                                    .user
                                    .lastName
                                    .toString(),
                            style: TextStyle(fontSize: ScreenUtil().setSp(12)),
                          ),
                        ),
                        Padding(
                          padding:
                              EdgeInsets.only(top: ScreenUtil().setWidth(9)),
                          child: Text(
                            " -" +
                                timeago.format(fifteenAgo, locale: 'en_short'),
                            style: TextStyle(
                                color: Color(0xff98A9BC),
                                fontSize: ScreenUtil().setSp(12)),
                          ),
                        )
                      ]),
                      Padding(
                        padding:
                            EdgeInsets.only(left: ScreenUtil().setWidth(15)),
                        child: Text(this.widget.postCommentModel.user.about,
                            style: TextStyle(
                                color: Color(0xff98A9BC),
                                fontSize: ScreenUtil().setSp(12))),
                      ),
                      Text(""),
                      Padding(
                        padding: EdgeInsets.only(
                          bottom: ScreenUtil().setWidth(8),
                          left: ScreenUtil().setWidth(15),
                          right: ScreenUtil().setWidth(15),
                        ),
                        child: Text(
                          this.widget.postCommentModel.comment,
                          style: TextStyle(fontSize: ScreenUtil().setSp(14)),
                        ),
                      ),
                    ],
                  ),
                ),
              ],
            ),
          ),
        ],
      ),
    );
  }
}
