import 'package:feed_plugin/bloc/feed_state.dart';
import 'package:feed_plugin/models/user_model.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:flutter_screenutil/screenutil_init.dart';
import 'package:feed_plugin/models/report_sub_type_model.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter/material.dart';
import 'dart:async';
import 'package:intl/intl.dart';
import 'package:datetime_picker_formfield/datetime_picker_formfield.dart';
import 'package:feed_plugin/bloc/feed_cubit.dart';
import 'package:feed_plugin/models/post_model.dart';
import 'package:feed_plugin/pages/my_feed.dart';
import 'package:feed_plugin/pages/partner_feed.dart';
import 'package:feed_plugin/pages/share_post_page.dart';
import 'package:feed_plugin/widgets/loadingButtonIndicator.dart';
import 'package:feed_plugin/widgets/post.dart';
import 'package:pusher_websocket_flutter/pusher.dart';
import 'dart:convert';
import '../environment_config.dart';
import 'package:empty_widget/empty_widget.dart';

final lastDate = new DateTime(
  DateTime.now().year - 10,
  DateTime.now().month,
);

String selectedValue = lastDate.toString();

class HomePage extends StatefulWidget {
  const HomePage(
      {Key key,
      this.token,
      this.firstName,
      this.lastName,
      this.about,
      this.profileImg,
      this.partnerName,
      this.partnerProfileImg,
      this.partnerMedia,
      this.partnerToken,
      this.currentPartner,
      this.currentUser})
      : super(key: key);

  final String token;
  final String firstName;
  final String lastName;
  final String about;
  final String profileImg;
  final String partnerName;
  final String partnerProfileImg;
  final String partnerToken;
  final UserModel currentPartner;
  final UserModel currentUser;
  final partnerMedia;

  @override
  _HomePageState createState() => _HomePageState();
}

class _HomePageState extends State<HomePage> {
  FeedCubit feedCubit;
  List<PostModel> postsList = [];
  List<PostModel> postsListFilter = [];
  UserModel currentPartner;
  UserModel currentUser;
  bool newPostFlag = false;
  Channel _channel;
  var _scrollController = ScrollController();
  double defaultScreenWidth = 393.0;
  double defaultScreenHeight = 534.0;
  int likeId;
  int dislikeId;
  int saveId;
  bool flagFilter;
  int count;

  @override
  void initState() {
    feedCubit = BlocProvider.of<FeedCubit>(context);
    flagFilter = false;
    count = 0;
    getCurrentUserAndPartner();
    initPusher();
    getPosts();
    super.initState();
  }

  dispose() {
    super.dispose();
  }

  getCurrentUserAndPartner() async {
    if (this.widget.currentPartner != null || this.widget.currentUser != null) {
      feedCubit.fetchPartnerAndUserData(
          this.widget.currentPartner, this.widget.currentUser);
    } else {
      if (this.widget.partnerToken != null) {
        await feedCubit.fetchPartnerInfo(
            this.widget.partnerName,
            this.widget.partnerProfileImg,
            this.widget.partnerToken,
            this.widget.partnerMedia);
      }
      await feedCubit.fetchUserInfo(this.widget.firstName, this.widget.lastName,
          this.widget.about, this.widget.profileImg, this.widget.token);
    }
  }

  Future<void> initPusher() async {
    try {
      await Pusher.init(
          EnvironmentConfig.PUSHER_ENV, PusherOptions(cluster: 'ap2'));
    } catch (e) {
      print(e);
    }

    Pusher.connect(onConnectionStateChange: (val) {}, onError: (err) {});

    _channel = await Pusher.subscribe('post');
    _channel.bind('create_post', (onEvent) {
      List<dynamic> newPost = json.decode(onEvent.data);
      List<PostModel> postsList =
          newPost.map((i) => PostModel.fromJson(i)).toList();
      if (mounted) {
        this.setState(() {
          postsListFilter.add(postsList[0]);
          newPostFlag = true;
        });
      }
    });

    _channel.bind('remove_post', (onEvent) {
      int postId = int.parse(onEvent.data);
      List<PostModel> posts = [];
      postsListFilter.forEach((post) {
        if (post.id != postId) {
          posts.add(post);
        }
      });
      if (mounted) {
        this.setState(() {
          postsListFilter = posts;
        });
      }
    });
  }

  getPosts() async {
    await getPostsReports();
    await loadPosts();
  }

  Future getPostsReports() async {
    List<ReportSubTypeModel> reportsSubTypeList =
        await feedCubit.feedRepository.fetchAllSubType();
    for (var i = 0; i < reportsSubTypeList.length; i++) {
      if (likeId == null || dislikeId == null || saveId == null) {
        if (reportsSubTypeList[i].type == "Like") {
          likeId = reportsSubTypeList[i].id;
        } else if (reportsSubTypeList[i].type == "Dislike") {
          dislikeId = reportsSubTypeList[i].id;
        } else if (reportsSubTypeList[i].type == "Save") {
          saveId = reportsSubTypeList[i].id;
        }
      } else {
        break;
      }
    }
  }

  void startTimer(int num) {
    count = num;
    Timer.periodic(Duration(seconds: 1), (_timer) {
      if (count > 0) {
        count--;
      } else {
        _timer.cancel();
        setState(() {
          newPostFlag = false;
        });
      }
    });
  }

  bool filter({@required DateTime createdAt, @required String selectedDate}) {
    DateTime fromDate = createdAt;
    DateTime toDate = new DateFormat("yyyy-MM-dd hh:mm:ss").parse(selectedDate);
    if (fromDate.isAfter(toDate) || toDate == fromDate) {
      return true;
    }
    return false;
  }

  bool filterByDay(
      {@required DateTime createdAt, @required String selectedDate}) {
    DateTime theDate =
        new DateFormat("yyyy-MM-dd hh:mm:ss").parse(selectedDate);
    setState(() {
      flagFilter = true;
    });
    if (theDate.day == createdAt.day &&
        theDate.month == createdAt.month &&
        theDate.year == createdAt.year) {
      return true;
    }
    return false;
  }

  void getPostsByFilter() {
    List<PostModel> postsListTemp = [];

    for (var i = 0; i < postsList.length; i++) {
      if (filterByDay(
          createdAt: postsList[i].createdAt, selectedDate: selectedValue))
      // filter(createdAt: postsList[i].createdAt, selectedDate: selectedValue))
      {
        postsListTemp.add(postsList[i]);
      }
    }
    setState(
      () {
        postsListFilter = postsListTemp;
      },
    );
  }

  Future<void> loadPosts() async {
    List<PostModel> posts = await feedCubit.feedRepository.loadPosts();
    this.setState(() {
      postsListFilter = posts;
      postsList = posts;
    });
    // return posts;
  }

  Future refreshData() async {
    feedCubit = BlocProvider.of<FeedCubit>(context);
    await getPosts();
    return;
  }

  @override
  Widget build(BuildContext context) {
    return ScreenUtilInit(
      designSize: Size(defaultScreenWidth, defaultScreenHeight),
      allowFontScaling: true,
      builder: () => LayoutBuilder(
        builder: (_, BoxConstraints constraints) {
          return MaterialApp(
            home: Scaffold(
              appBar: PreferredSize(
                  preferredSize: Size.fromHeight(70.0),
                  child: AppBar(
                    backgroundColor: Color(0xFF083356),
                    actions: <Widget>[
                      Container(
                        child: BlocBuilder<FeedCubit, FeedState>(
                            builder: (context, state) {
                          if (state is FeedReadyState) {
                            if (state.currentPartner != null &&
                                state.currentUser != null) {
                              currentPartner = state.currentPartner;
                              currentUser = state.currentUser;
                              return GestureDetector(
                                  onTap: () {
                                    Navigator.pop(context);
                                  },
                                  child: GestureDetector(
                                      onTap: () {
                                        Navigator.push(
                                            context,
                                            MaterialPageRoute(
                                                builder: (context) =>
                                                    PartnerFeedPage(
                                                      currentPartner:
                                                          state.currentPartner,
                                                      currentUser:
                                                          state.currentUser,
                                                      saveId: saveId,
                                                      likeId: likeId,
                                                      dislikeId: dislikeId,
                                                    )));
                                      },
                                      child: Padding(
                                        padding: EdgeInsets.only(
                                            right: ScreenUtil().setWidth(10)),
                                        child: Container(
                                            alignment: Alignment.bottomRight,
                                            child: ClipOval(
                                                child: Image.network(
                                              state.currentPartner.profileImg,
                                              width: ScreenUtil().setWidth(40),
                                              height: ScreenUtil().setWidth(40),
                                              fit: BoxFit.cover,
                                            ))),
                                      )));
                            }
                          }
                          return Padding(
                              padding: EdgeInsets.only(
                                  right: ScreenUtil().setWidth(10)),
                              child: Container());
                        }),
                      ),
                    ],
                    leading: BlocBuilder<FeedCubit, FeedState>(
                        builder: (context, feedState) {
                      if (feedState is FeedReadyState) {
                        if (feedState.currentUser != null) {
                          return GestureDetector(
                              onTap: () {
                                Navigator.pop(context);
                              },
                              child: GestureDetector(
                                  onTap: () {
                                    Navigator.push(
                                        context,
                                        MaterialPageRoute(
                                            builder: (context) => MyFeedPage(
                                                  currentPartner:
                                                      currentPartner,
                                                  currentUser:
                                                      feedState.currentUser,
                                                  saveId: saveId,
                                                  likeId: likeId,
                                                  dislikeId: dislikeId,
                                                )));
                                  },
                                  child: Container(
                                      alignment: Alignment.bottomRight,
                                      child: feedState.currentUser.profileImg !=
                                              null
                                          ? ClipOval(
                                              child: Image.network(
                                              feedState.currentUser.profileImg,
                                              width: ScreenUtil().setWidth(40),
                                              height: ScreenUtil().setWidth(40),
                                              fit: BoxFit.cover,
                                            ))
                                          : Icon(Icons.account_circle_outlined,
                                              size: 35, color: Colors.white))));
                        }
                      }

                      return Container();
                    }),
                    title: Padding(
                      padding: EdgeInsets.only(
                          left: ScreenUtil().setWidth(50),
                          top: ScreenUtil().setWidth(10)),
                      child: Text(
                        "Phone.do Feed",
                        style: TextStyle(color: Colors.white),
                      ),
                    ),
                  )),
              body: postsListFilter.length < 1 && flagFilter
                  ? GestureDetector(
                      onTap: () {
                        setState(() {
                          postsListFilter = postsList;
                          flagFilter = false;
                        });
                      },
                      child: Center(
                        child: EmptyListWidget(
                            title: 'No Data Found',
                            subTitle: 'Click here to see all posts',
                            packageImage: PackageImage.Image_3,
                            titleTextStyle: Theme.of(context)
                                .typography
                                .dense
                                .headline4
                                .copyWith(color: Color(0xff9da9c7)),
                            subtitleTextStyle: Theme.of(context)
                                .typography
                                .dense
                                .bodyText1
                                .copyWith(color: Color(0xffabb8d6))),
                      ),
                    )
                  : postsListFilter.length < 1 && !flagFilter
                      ? Center(child: LoadingIndicatorDark())
                      : Column(
                          crossAxisAlignment: CrossAxisAlignment.stretch,
                          children: [
                            _buildDateField(),
                            Expanded(child: BlocBuilder<FeedCubit, FeedState>(
                                builder: (context, feedState) {
                              if (feedState is FeedReadyState) {
                                if (feedState.currentUser != null) {
                                  return Stack(
                                    children: [
                                      RefreshIndicator(
                                        onRefresh: () {
                                          return refreshData();
                                        },
                                        child: ListView.separated(
                                          cacheExtent: 9999,
                                          controller: _scrollController,
                                          itemCount: postsListFilter.length,
                                          itemBuilder: (BuildContext context,
                                              int index) {
                                            PostModel post =
                                                postsListFilter[index];
                                            List<String> images =
                                                postsListFilter[index]
                                                    .attachedUrl
                                                    .split(",");
                                            return Container(
                                              decoration: BoxDecoration(
                                                  border: Border(
                                                      bottom: BorderSide(
                                                          width: 15,
                                                          color: Color(
                                                              0xffE8ECEF)))),
                                              child: FeedPost(
                                                postData: post,
                                                currentUser:
                                                    feedState.currentUser,
                                                images: images,
                                                likeId: likeId,
                                                dislikeId: dislikeId,
                                                saveId: saveId,
                                                commentsLength: 0,
                                              ),
                                            );
                                          },
                                          separatorBuilder:
                                              (BuildContext context,
                                                      int index) =>
                                                  const Divider(),
                                        ),
                                      ),
                                      if (newPostFlag == true) _newPostButton(),
                                      Align(
                                        alignment: Alignment(0.8, 0.85),
                                        child: Container(
                                            decoration: BoxDecoration(
                                              shape: BoxShape.circle,
                                              boxShadow: [
                                                BoxShadow(
                                                  color: Colors.grey,
                                                  blurRadius: 5.0,
                                                  spreadRadius: 1.0,
                                                )
                                              ],
                                            ),
                                            child: ClipOval(
                                                child: Material(
                                                    color: Color(0xFF3DD598),
                                                    child: InkWell(
                                                      splashColor:
                                                          Color(0xFFFCA64F),
                                                      child: SizedBox(
                                                        width: 70,
                                                        height: 70,
                                                        child: Icon(Icons.add,
                                                            size: 50.0,
                                                            color:
                                                                Colors.white),
                                                      ),
                                                      onTap: () {
                                                        Navigator.push(
                                                          context,
                                                          MaterialPageRoute(
                                                              builder: (context) =>
                                                                  SharePostPage(
                                                                      currentUser:
                                                                          feedState
                                                                              .currentUser)),
                                                        );
                                                      },
                                                    )))),
                                      )
                                    ],
                                  );
                                }
                              }
                              return Container();
                            })),
                          ],
                        ),
            ),
          );
        },
      ),
    );
  }

  _newPostButton() {
    startTimer(7);
    return Container(
        height: ScreenUtil().setHeight(40),
        alignment: Alignment.center,
        child: SizedBox(
          height: ScreenUtil().setHeight(30),
          width: ScreenUtil().setHeight(170),
          child: Container(
            decoration: ShapeDecoration(
                color: Color(0xFF3DD598),
                shape: RoundedRectangleBorder(
                  side: BorderSide(
                      color: Colors.transparent,
                      width: 1,
                      style: BorderStyle.solid),
                  borderRadius: BorderRadius.circular(25),
                )),
            //ignore: deprecated_member_use
            child: FlatButton(
              onPressed: () {
                startTimer(0);
                _scrollController.animateTo(
                    _scrollController.position.maxScrollExtent,
                    duration: Duration(seconds: 1),
                    curve: Curves.fastOutSlowIn);
              },
              child: Padding(
                padding: EdgeInsets.only(
                  left: ScreenUtil().setWidth(70),
                ),
                child: Row(
                  children: [
                    Text('New Post', style: TextStyle(color: Colors.white)),
                    Icon(
                      Icons.arrow_forward,
                      size: 35,
                      color: Colors.white,
                    )
                  ],
                ),
              ),
            ),
          ),
        ));
  }

  _buildDateField() {
    final format = DateFormat("dd/MM/yyyy");
    final TextEditingController controller = TextEditingController();
    final now = DateTime.now();

    final nextYear = new DateTime(
      now.year,
      now.month,
      now.day,
    );
    return Container(
        decoration: BoxDecoration(
            border: Border(
          top: BorderSide(width: 15, color: Color(0xffE8ECEF)),
          bottom: BorderSide(width: 15, color: Color(0xffE8ECEF)),
        )),
        height: ScreenUtil().setHeight(70),
        child: Padding(
          padding: EdgeInsets.all(ScreenUtil().setWidth(15)),
          child: Container(
            decoration: BoxDecoration(
                color: Color(0xffE8ECEF),
                borderRadius: BorderRadius.circular(7)),
            margin: EdgeInsets.only(top: ScreenUtil().setHeight(5)),
            child: DateTimeField(
              textAlign: TextAlign.center,
              readOnly: true,
              onChanged: (value) {
                setState(
                  () {
                    selectedValue = value.toString();
                  },
                );
                getPostsByFilter();
              },
              controller: controller,
              format: format,
              onShowPicker: (context, currentValue) {
                return showDatePicker(
                    context: context,
                    firstDate: lastDate,
                    initialDate: currentValue ?? DateTime.now(),
                    lastDate: nextYear);
              },
              decoration: new InputDecoration(
                  labelText: " filter...",
                  labelStyle: TextStyle(color: Color(0xff98A9BC))),
            ),
          ),
        ));
  }
}
