import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:feed_plugin/bloc/feed_cubit.dart';
import 'package:feed_plugin/models/post_model.dart';
import 'package:feed_plugin/models/report_sub_type_model.dart';
import 'package:feed_plugin/models/report_type_model.dart';
import 'package:feed_plugin/models/user_model.dart';
import 'package:rflutter_alert/rflutter_alert.dart';
import 'package:status_alert/status_alert.dart';

List<ReportTypeModel> reportsList = [];
List<ReportSubTypeModel> reportsSubTypesList = [];
int flagState = 0;

FeedCubit feedCubit;

class Reports extends StatefulWidget {
  const Reports({
    Key key,
    @required this.postData,
    @required this.currentUser,
    @required this.onClick,
  }) : super(key: key);

  final PostModel postData;
  final UserModel currentUser;
  final VoidCallback onClick;

  @override
  _ReportsState createState() => _ReportsState();
}

class _ReportsState extends State<Reports> {
  FeedCubit feedCubit;

  void initState() {
    feedCubit = BlocProvider.of<FeedCubit>(context);
    super.initState();
  }

  Future<void> reportPost(subTypeId) async {
    await feedCubit.feedRepository.addReport(
        subTypeId, this.widget.currentUser.id, this.widget.postData.id);
    await feedCubit.feedRepository.updatePostReport(this.widget.postData.id);
  }

  Future<List<ReportTypeModel>> getReportsList() async {
    reportsList = await feedCubit.feedRepository.fetchAllReports();
    return reportsList;
  }

  Future<List<ReportSubTypeModel>> getSubTypesReportsList() async {
    reportsSubTypesList =
        await feedCubit.feedRepository.fetchSubReport(flagState);
    return reportsSubTypesList;
  }

  Widget build(BuildContext context) {
    return FutureBuilder(
        future: getReportsList(),
        builder: (context, reportSnap) {
          if (reportSnap.connectionState == ConnectionState.waiting ||
              reportSnap.connectionState == ConnectionState.none) {
            return Center(child: CircularProgressIndicator());
          }
          return FutureBuilder(
              future: getSubTypesReportsList(),
              builder: (context, reportSubSnap) {
                if (reportSubSnap.connectionState == ConnectionState.waiting ||
                    reportSubSnap.connectionState == ConnectionState.none) {
                  return Center(child: CircularProgressIndicator());
                }

                return AlertDialog(
                    title: Column(
                      children: [
                        Row(
                          children: [
                            flagState != 0
                                ? TextButton(
                                    child: Container(
                                      alignment: Alignment.centerLeft,
                                      width: ScreenUtil().setWidth(160),
                                      child: Icon(
                                        Icons.chevron_left,
                                        color: Colors.black,
                                      ),
                                    ),
                                    onPressed: () {
                                      setState(() {
                                        flagState = 0;
                                        setState(() {});
                                      });
                                      setupAlertDialoadContainer();
                                    },
                                  )
                                : Container(
                                    alignment: Alignment.centerLeft,
                                    width: ScreenUtil().setWidth(170),
                                    child: Text("")),
                            TextButton(
                              child: Container(
                                alignment: Alignment.centerRight,
                                child: Icon(
                                  Icons.close,
                                  color: Colors.black,
                                ),
                              ),
                              onPressed: () {
                                setState(() {
                                  flagState = 0;
                                  setState(() {});
                                });
                                Navigator.of(context).pop();
                              },
                            ),
                          ],
                        ),
                        flagState == 0
                            ? Text(
                                'why are you reporting this?',
                                style: TextStyle(
                                    color: Colors.black,
                                    fontSize: ScreenUtil().setSp(21)),
                              )
                            : Container(
                                width: ScreenUtil().setWidth(300),
                                child: Text(
                                  'Tell us a little more',
                                  style: TextStyle(
                                      color: Colors.black,
                                      fontSize: ScreenUtil().setSp(22)),
                                ),
                              ),
                        Divider(
                          indent: 7,
                          endIndent: 2,
                          height: 20,
                          color: Colors.black,
                        ),
                      ],
                    ),
                    content: (flagState == 0)
                        ? setupAlertDialoadContainer()
                        : setupAlertSubReportsTyps());
              });
        });
  }

  setupAlertDialoadContainer() {
    return Container(
      height: 160.0,
      width: 200.0,
      child: ListView.builder(
        itemCount: reportsList.length,
        itemBuilder: (BuildContext context, int index) {
          return (reportsList[index].type == "Likes")
              ? Text("")
              : Row(
                  children: [
                    Container(
                      child: DialogButton(
                        onPressed: () {
                          setState(() {
                            flagState = reportsList[index].id;
                            setState(() {});
                          });
                        },
                        color: Colors.white,
                        child: Container(
                          height: ScreenUtil().setHeight(50),
                          child: Text(
                            "I thinnk it’s " +
                                reportsList[index].type.toString(),
                            style: TextStyle(
                                color: Colors.black,
                                fontSize: ScreenUtil().setSp(18)),
                          ),
                        ),
                      ),
                    ),
                  ],
                );
        },
      ),
    );
  }

  Widget setupAlertSubReportsTyps() {
    return Container(
      height: 160.0,
      width: 340.0,
      child: ListView.builder(
        itemCount: reportsSubTypesList.length,
        itemBuilder: (BuildContext context, int index) {
          return Row(
            children: [
              Container(
                child: DialogButton(
                  onPressed: () {
                    StatusAlert.show(
                      context,
                      duration: Duration(seconds: 2),
                      title: 'Thanks for your Report',
                      configuration: IconConfiguration(
                        icon: Icons.add_moderator,
                        size: 50,
                      ),
                    );
                    flagState = 0;
                    this.widget.onClick();
                    reportPost(reportsSubTypesList[index].id);
                    Navigator.of(context).popAndPushNamed("/homePage");
                  },
                  color: Colors.white,
                  child: Text(
                    reportsSubTypesList[index].type.toString(),
                    style: TextStyle(
                        color: Colors.black, fontSize: ScreenUtil().setSp(18)),
                  ),
                ),
              ),
            ],
          );
        },
      ),
    );
  }
}
